"use strict";

var Promise = require("bluebird");

var IdentifiedElement = require('./IdentifiedElement');
var LayoutAlias = require('./LayoutAlias');
var LayoutReaction = require('./LayoutReaction');
var logger = require('../../logger');

/**
 * Class representing data in a specific layout.
 */
function DataOverlay(layoutId, name) {
  this.setInitialized(false);

  if (name === undefined) {
    // from json structure
    var object = layoutId;
    this.setId(object.idObject);
    this.setOrder(object.order);
    this.setName(object.name);
    this.setImagesDirectory(object.images);
    this.setDescription(object.description);
    this.setCreator(object.creator);
    this.setContent(object.content);
    this.setFilename(object.filename);
    this.setPublicOverlay(object.publicOverlay);
    this.setDefaultOverlay(object.defaultOverlay);
    this.setInputDataAvailable(object.inputDataAvailable);
    this.setType(object.type);
    if (!this.getInputDataAvailable()) {
      this.setInitialized(true);
    }
  } else {
    // default two param call
    this.setId(layoutId);
    this.setName(name);
  }
  this.aliases = [];
  this.aliasById = [];
  this.reactions = [];
}

/**
 * Adds alias to the {@link DataOverlay}
 *
 * @param layoutAlias
 *          information about alias in a layout
 */
DataOverlay.prototype.addAlias = function (layoutAlias) {
  this.aliases.push(layoutAlias);
  this.aliasById[layoutAlias.getId()] = layoutAlias;
};

/**
 * Adds reaction to the {@link DataOverlay}
 *
 * @param layoutReaction
 *          information about reaction in a layout
 */
DataOverlay.prototype.addReaction = function (layoutReaction) {
  this.reactions.push(layoutReaction);
};

DataOverlay.prototype.getId = function () {
  return this.id;
};

DataOverlay.prototype.setId = function (id) {
  this.id = parseInt(id);
};

DataOverlay.prototype.getDescription = function () {
  return this._description;
};

DataOverlay.prototype.setDescription = function (description) {
  this._description = description;
};

DataOverlay.prototype.getCreator = function () {
  return this._creator;
};

DataOverlay.prototype.setCreator = function (creator) {
  this._creator = creator;
};

DataOverlay.prototype.getInputDataAvailable = function () {
  return this._inputDataAvailable;
};

DataOverlay.prototype.setInputDataAvailable = function (inputDataAvailable) {
  var value = inputDataAvailable;
  if (inputDataAvailable === undefined) {
    value = false;
  } else if (inputDataAvailable === "true") {
    value = true;
  } else if (inputDataAvailable === "false") {
    value = false;
  } else if (typeof (inputDataAvailable) !== "boolean") {
    logger.warn("inputDataAvailable should be boolean");
    value = false;
  }
  this._inputDataAvailable = value;
};

DataOverlay.prototype.getName = function () {
  return this.name;
};

DataOverlay.prototype.setName = function (name) {
  this.name = name;
};

DataOverlay.prototype.getImagesDirectory = function (modelId) {
  for (var i = 0; i < this._imagesDirectory.length; i++) {
    if (parseInt(this._imagesDirectory[i].modelId) === modelId) {
      return this._imagesDirectory[i].path;
    }
  }
  return null;
};

DataOverlay.prototype.setImagesDirectory = function (imagesDirectory) {
  this._imagesDirectory = imagesDirectory;
};

DataOverlay.prototype.updateAlias = function (layoutAlias) {
  if (this.aliasById[layoutAlias.getId()] === undefined) {
    logger.warn("Cannot update alias, it doesn't exist. Alias: ", layoutAlias.getId());
  } else {
    this.aliasById[layoutAlias.getId()].update(layoutAlias);
  }

};

DataOverlay.prototype.getAliasById = function (id) {
  return this.aliasById[id];
};

DataOverlay.prototype.getFullAliasById = function (id) {
  var self = this;
  var alias = self.getAliasById(id);
  if (alias !== undefined) {
    if (alias.getType() === LayoutAlias.LIGTH) {
      return ServerConnector.getFullOverlayElement({
        element: new IdentifiedElement(alias),
        overlay: self
      }).then(function (data) {
        self.updateAlias(data);
        return alias;
      });
    }
  }
  return Promise.resolve(alias);
};

DataOverlay.prototype.setInitialized = function (value) {
  this._initialized = value;
};

DataOverlay.prototype.isInitialized = function () {
  return this._initialized;
};

DataOverlay.prototype.getAliases = function () {
  return this.aliases;
};

DataOverlay.prototype.getReactions = function () {
  return this.reactions;
};

DataOverlay.prototype.init = function () {
  var self = this;
  if (this.isInitialized()) {
    return Promise.resolve();
  }
  return ServerConnector.getOverlayElements(self.getId()).then(function (data) {
    for (var i = 0; i < data.length; i++) {
      if (data[i] instanceof LayoutAlias) {
        self.addAlias(data[i]);
      } else if (data[i] instanceof LayoutReaction) {
        self.addReaction(data[i]);
      } else {
        return Promise.reject("Unknown element type: " + typeof (data[i]));
      }

    }
    self.setInitialized(true);
  });

};

DataOverlay.prototype.getPublicOverlay = function () {
  return this._publicOverlay;
};

DataOverlay.prototype.setPublicOverlay = function (publicOverlay) {
  this._publicOverlay = publicOverlay;
};

DataOverlay.prototype.isDefaultOverlay = function () {
  return this._defaultOverlay;
};

DataOverlay.prototype.setDefaultOverlay = function (defaultOverlay) {
  this._defaultOverlay = defaultOverlay;
};

DataOverlay.prototype.getContent = function () {
  return this._content;
};

DataOverlay.prototype.setContent = function (content) {
  this._content = content;
};

DataOverlay.prototype.getOrder = function () {
  return this._order;
};

DataOverlay.prototype.setOrder = function (order) {
  this._order = order;
};

DataOverlay.prototype.getFilename = function () {
  return this._filename;
};

DataOverlay.prototype.setFilename = function (filename) {
  this._filename = filename;
};

DataOverlay.prototype.getType = function () {
  return this._type;
};

DataOverlay.prototype.setType = function (type) {
  this._type = type;
};

DataOverlay.prototype.update = function (overlay) {
  this.setName(overlay.getName());
  this.setDescription(overlay.getDescription());
};


module.exports = DataOverlay;
