"use strict";

var GuiConnector = require('../../GuiConnector');

var logger = require('../../logger');
var Promise = require("bluebird");

var ObjectWithListeners = require('../../ObjectWithListeners');

/**
 * Class representing Marker (called sometimes bubble) that we visualize on the
 * map.
 */
function AbstractMarker(params) {
  var self = this;
  // call super constructor
  ObjectWithListeners.call(this);

  self._icons = [];
  self.setIdentifiedElement(params.element);
  self.setIcon(params.element.getIcon());
  self.setCustomMap(params.map);

  self.registerListenerType("onClick");
  if (params.onClick !== undefined) {
    if (typeof params.onClick !== "function") {
      for (var i = 0; i < params.onClick.length; i++) {
        self.addListener("onClick", params.onClick[i]);
      }
    } else {
      self.addListener("onClick", params.onClick);
    }
  }

}

AbstractMarker.prototype = Object.create(ObjectWithListeners.prototype);
AbstractMarker.prototype.constructor = AbstractMarker;

/**
 * Returns identifier of this object.
 *
 * @returns identifier of this object
 */
AbstractMarker.prototype.getId = function () {
  return this.getIdentifiedElement().getId();
};

/**
 * Returns icon of the marker.
 *
 * @returns string/undefined identifying icon of the marker
 */
AbstractMarker.prototype.getIcon = function () {
  var icons = this._icons;
  if (icons.length === 0) {
    return undefined;
  } else if (icons.length === 1) {
    return icons[0];
  } else {
    var result = icons[0];
    //if we have many identical icons then return this specific icon
    for (var i = 1; i < icons.length; i++) {
      if (icons[i] !== result) {
        return "marker/multi.png";

      }
    }
    return result;
  }
};

AbstractMarker.prototype._updateIcon = function () {
  var self = this;
  var googleMarker = self.getGoogleMarker();
  if (googleMarker !== undefined) {
    if (self.getIcon() === undefined) {
      self.hide();
    } else {
      googleMarker.setIcon(GuiConnector.getImgPrefix() + self.getIcon());
      if (googleMarker.getMap() === null) {
        self.show();
      }
    }
  }
};

AbstractMarker.prototype.setIcon = function (icon) {
  var self = this;
  if (icon === null || icon === undefined) {
    self._icons = [];
  } else {
    self._icons = [icon];
  }
  self._updateIcon(self);
};

AbstractMarker.prototype.setIcons = function (icons) {
  var self = this;
  self._icons = icons;
  self._updateIcon(self);
};

AbstractMarker.prototype.addIcon = function (icon) {
  var self = this;
  self._icons.push(icon);
  self._updateIcon(self);
};
AbstractMarker.prototype.addIcons = function (icons) {
  var self = this;
  for (var i = 0; i < icons.length; i++) {
    self._icons.push(icons[i]);
  }
  self._updateIcon(self);
};
AbstractMarker.prototype.removeIcons = function (icons) {
  var self = this;
  for (var i = 0; i < icons.length; i++) {
    self.removeIcon(icons[i], true);
  }
  self._updateIcon(self);
};

AbstractMarker.prototype.removeIcon = function (icon, preventMarkerUpdate) {
  var self = this;
  var index = self._icons.indexOf(icon);

  if (index > -1) {
    self._icons.splice(index, 1);
    if (!preventMarkerUpdate) {
      self._updateIcon(self);
    }
  } else {
    logger.warn("Unknown icon for marker: ", icon);
  }
};

/**
 * Shows marker on the map.
 */
AbstractMarker.prototype.show = function () {
  if (this.getGoogleMarker() === undefined) {
    logger.warn("Cannot show marker. Marker not initialized");
    return;
  }
  if (this.getGoogleMarker().getMap() !== undefined && this.getGoogleMarker().getMap() !== null) {
    logger.warn("Marker is already shown");
  } else {
    this.getGoogleMarker().setMap(this.getCustomMap().getGoogleMap());
  }
};
AbstractMarker.prototype.hide = function () {
  if (this.getGoogleMarker() === undefined) {
    logger.warn("Cannot hide marker. Marker not initialized");
    return;
  }
  if (this.getGoogleMarker().getMap() === null || this.getGoogleMarker().getMap() === undefined) {
    logger.warn("Marker is already invisible");
  } else {
    this.getGoogleMarker().setMap(null);
  }
};

AbstractMarker.prototype.isShown = function () {
  if (this.getGoogleMarker() === undefined) {
    return false;
  }
  return (this.getGoogleMarker().getMap() !== undefined && this.getGoogleMarker().getMap() !== null);
};

/**
 * Returns {@link AbstractCustomMap} where marker is located.
 *
 * @returns AbstractCustomMap where marker is located
 */
AbstractMarker.prototype.getCustomMap = function () {
  return this._map;
};

AbstractMarker.prototype.setCustomMap = function (map) {
  this._map = map;
};

/**
 * Returns google.maps.LatLngBounds of the marker (it's a single point).
 *
 * @returns google.maps.LatLngBounds bounds of the marker (it's a single point)
 */
AbstractMarker.prototype.getBounds = function () {
  var marker = this.getGoogleMarker();
  if (marker === undefined || marker === null) {
    logger.warn("Marker not initialized");
    return null;
  }
  var bounds = new google.maps.LatLngBounds();
  bounds.extend(marker.position);

  return bounds;
};

/**
 * Set map where the marker should be visualized.
 *
 * @param map
 *          {@link google.maps.Map} where the marker should be visualized.
 */
AbstractMarker.prototype.setMap = function (map) {
  if (this.getGoogleMarker() !== undefined) {
    this.getGoogleMarker().setMap(map);
  } else {
    logger.warn("[" + this.constructor.name + "] Cannot setup map. Marker wasn't initialized. Marker id: "
      + this.getId());
  }
};

/**
 * Initializes google.maps.Marker object connected to this object.
 */
AbstractMarker.prototype._init = function () {
  var self = this;
  var point = this.getCoordinates();
  var latLng = this.getCustomMap().fromPointToLatLng(point);

  this._marker = new google.maps.Marker({
    position: latLng,
    icon: GuiConnector.getImgPrefix() + self.getIcon(),
    optimized: self.getCustomMap().isMarkerOptimization(),
    id: self.getId()
  });

  var onclick = function () {
    return self.onClickHandler().then(null, GuiConnector.alert);
  };
  google.maps.event.addListener(this.getGoogleMarker(), 'click', onclick);
  return Promise.resolve();
};

AbstractMarker.prototype.onClickHandler = function () {
  return this.callListeners("onClick");
};

/**
 * Returns {google.maps.Marker} connected to this object.
 *
 * @returns {google.maps.Marker} connected to this object
 */
AbstractMarker.prototype.getGoogleMarker = function () {
  return this._marker;
};

/**
 * Abstract function returning string marker type.
 *
 * @returns string marker type
 */
AbstractMarker.prototype.getType = function () {
  return this.getIdentifiedElement().getType();
};

AbstractMarker.prototype.getModelId = function () {
  return this.getIdentifiedElement().getModelId();
};

AbstractMarker.prototype.getIdentifiedElement = function () {
  return this._identifiedElement;
};

AbstractMarker.prototype.setIdentifiedElement = function (identifiedElement) {
  this._identifiedElement = identifiedElement;
};

module.exports = AbstractMarker;
