"use strict";

var logger = require('../../logger');

var ObjectWithListeners = require('../../ObjectWithListeners');

/**
 * Class representing abstract overlay element on the map relevant for a
 * specific layout.
 */
function AbstractSurfaceElement(params) {
  var self = this;
  // call super constructor
  ObjectWithListeners.call(this);

  self.registerListenerType("onClick");
  self.setCustomMap(params.map);
  if (params.element !== undefined && params.element !== null) {
    self.setIdentifiedElement(params.element);
  }
  self._googleMapObjects = [];

  if (params.onClick !== undefined) {
    if (typeof params.onClick !== "function") {
      for (var i = 0; i < params.onClick.length; i++) {
        self.addListener("onClick", params.onClick[i]);
      }
    } else {
      self.addListener("onClick", params.onClick);
    }
  }
}

AbstractSurfaceElement.prototype = Object.create(ObjectWithListeners.prototype);
AbstractSurfaceElement.prototype.constructor = AbstractSurfaceElement;

AbstractSurfaceElement.prototype.setMap = function (map) {
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    this.getGoogleMapObjects()[i].setMap(map);
  }
};

AbstractSurfaceElement.prototype.getBounds = function () {
  var bounds = new google.maps.LatLngBounds();
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    bounds.extend(this.getGoogleMapObjects()[i].getBounds().getSouthWest());
    bounds.extend(this.getGoogleMapObjects()[i].getBounds().getNorthEast());
  }
  return bounds;
};

AbstractSurfaceElement.prototype.isShown = function () {
  var result = false;
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    var map = this.getGoogleMapObjects()[i].getMap();
    if (map !== null && map !== undefined) {
      result = true;
    }
  }

  return result;
};

AbstractSurfaceElement.prototype.show = function () {
  if (this.isShown()) {
    logger.warn("Surface already shown");
    return;
  }
  this.setMap(this.getCustomMap().getGoogleMap());
};

AbstractSurfaceElement.prototype.hide = function () {
  this.setMap(null);
};

AbstractSurfaceElement.prototype.onClickHandler = function () {
  return this.callListeners("onClick");
};

AbstractSurfaceElement.prototype.getGoogleMarker = function () {
  return this.getGoogleMapObjects()[0];
};

AbstractSurfaceElement.prototype.getGoogleMapObjects = function () {
  return this._googleMapObjects;
};

AbstractSurfaceElement.prototype.addGoogleMapObject = function (googleObject) {
  this._googleMapObjects.push(googleObject);

  var self = this;
  var onclick = function () {
    return self.onClickHandler();
  };
  google.maps.event.addListener(googleObject, 'click', onclick);
};

AbstractSurfaceElement.prototype.getIdentifiedElement = function () {
  return this._identifiedElement;
};

AbstractSurfaceElement.prototype.setIdentifiedElement = function (identifiedElement) {
  this._identifiedElement = identifiedElement;
};

AbstractSurfaceElement.prototype.getBioEntity = function () {
  return this._bioEntity;
};

AbstractSurfaceElement.prototype.setBioEntity = function (bioEntity) {
  this._bioEntity = bioEntity;
};


AbstractSurfaceElement.prototype.getModelId = function () {
  return this.getIdentifiedElement().getModelId();
};

AbstractSurfaceElement.prototype.updateIdentifiedElement = function (identifiedElement) {
  if (identifiedElement.getColor() !== undefined) {
    this.setColor(identifiedElement.getColor());
  }
};

AbstractSurfaceElement.prototype.getBounds = function () {
  var self = this;
  var bounds = new google.maps.LatLngBounds();
  for (var i = 0; i < self.getGoogleMapObjects().length; i++) {
    var elementBounds = self.getGoogleMapObjects()[i].getBounds();
    bounds.extend(elementBounds.getNorthEast());
    bounds.extend(elementBounds.getSouthWest());
  }

  return bounds;
};

/**
 * Returns {@link AbstractCustomMap} where surface is located.
 *
 * @returns {AbstractCustomMap} where surface is located
 */
AbstractSurfaceElement.prototype.getCustomMap = function () {
  return this._customMap;
};

AbstractSurfaceElement.prototype.setCustomMap = function (customMap) {
  this._customMap = customMap;
};

module.exports = AbstractSurfaceElement;
